#include "wlan.h"
#include <ArduinoJson.h>

void initial_connection(const char *ssid, const char *psk) {
  WiFi.begin(ssid, psk);
  WiFi.persistent(true);
  WiFi.setAutoConnect(true);
  WiFi.setAutoReconnect(true);
}

void connect_wlan(const char *ssid, const char *psk, const int max_retries,
                  const int sleep_time) {
  if (WiFi.SSID() != ssid)
    initial_connection(ssid, psk);
  int retries = 0;
  while (WiFi.status() != WL_CONNECTED) {
    if (retries == max_retries)
      enter_deep_sleep(true, sleep_time);
    retries++;
    delay(1000);
    Serial.print(".");
  }
  Serial.println("WiFi connected");
}

void connect_mqtt(PubSubClient &client, const char *mqtt_host,
                  const int mqtt_port, const char *mqtt_user,
                  const char *mqtt_password, const char *device_id,
                  const char *topic) {
  if (!client.connected())
    client.setServer(mqtt_host, mqtt_port);
  if (client.connect(device_id, mqtt_user, mqtt_password)) {
    Serial.println("MQTT connected");
    client.subscribe(topic);
  }
}

void disconnect_mqtt(PubSubClient &client, const char *topic) {
  Serial.println("Disconnecting MQTT");
  client.unsubscribe(topic);
  client.disconnect();
}

size_t construct_json(float *data, char *buffer, int buffer_size) {
  StaticJsonDocument<100> json;
  json["temperature"] = data[0];
  json["humidity"] = data[1];
  json["soil_humidity"] = data[2];
  size_t payload_size = serializeJson(json, buffer, buffer_size);
  return payload_size;
}

void mqtt_transfer(PubSubClient &client, const char *mqtt_host,
                   const int mqtt_port, const char *mqtt_user,
                   const char *mqtt_password, const char *device_id,
                   const char *topic, float *data) {
  char buffer[100];
  connect_mqtt(client, mqtt_host, mqtt_port, mqtt_user, mqtt_password,
               device_id, topic);
  size_t payload_size = construct_json(data, buffer, 100);
  client.publish(topic, buffer, payload_size);
  Serial.println("Data transferred successfully");
}

void enter_deep_sleep(bool wifi_timeout, int sleep_time) {
  Serial.println("Entering deep sleep");
  if (wifi_timeout)
    WiFi.disconnect();
  ESP.deepSleep(sleep_time, WAKE_RF_DEFAULT);
}
